from urllib.parse import parse_qs, urlparse

from reddit_radio.logging import logger


def extract_video_id(url):
    if url.startswith(("youtu", "www")):
        url = "http://" + url

    query = urlparse(url)
    hostname = query.hostname or ""
    video_id = None

    if "youtu" not in hostname:
        return None

    if hostname == "youtu.be":
        return query.path[1:]

    if "youtube.com" in hostname:
        if query.path == "/watch":
            video_id = parse_qs(query.query)["v"][0]
        elif query.path[:7] == "/embed/":
            video_id = query.path.split("/")[2]
        elif query.path[:3] == "/v/":
            video_id = query.path.split("/")[2]
        elif query.path == "/attribution_link":
            video_id = parse_qs(parse_qs(query.query)["u"][0].split("?")[1])["v"][0]
        elif query.path == "/playlist":
            video_id = parse_qs(query.query)["list"][0]

    if not video_id:
        logger.warning(f"Failed to get youtube id from [{url}]")

    return video_id



from itertools import chain
from pathlib import Path

import click

from reddit_radio import config
from reddit_radio.database import RedditPost, create_tables_if_needed
from reddit_radio.mpv import Client as MpvClient
from reddit_radio.reddit import Client as RedditClient


@click.group(chain=True)
def cli():
    pass


@cli.command()
@click.option("--count", default=100, help="Limit of tracks in the playlist")
def play(count):
    mpv = MpvClient()
    mpv.playlist(count)


@cli.command()
@click.option("--limit", default=100, help="Limit of posts per page")
@click.option("--pages", default=5, help="Number of pages to search")
def load_data(limit, pages):
    create_tables_if_needed()
    reddit = RedditClient()

    for post in chain.from_iterable(
        fn(sub, limit=limit, pages=pages)
        for sub in config.SUBREDDITS
        for fn in (reddit.hot, reddit.all)
    ):
        RedditPost.get_or_create(reddit_id=post["reddit_id"], defaults=post)


@cli.command()
@click.option("--force/--no-force", default=False, help="Override config if exists")
def make_config(force):
    config_file = config.config._config_file
    if not force and config_file.exists():
        click.echo("Config file found, skipping...")
        return

    template = Path(__file__).parent / "templates/config.sample.ini"
    config_file.parent.mkdir(exist_ok=True, parents=True)
    config_file.touch()
    config_file.write_text(template.read_text())
    click.echo(
        f"Config file created at {config_file}. Update the values before running."
    )



import configparser
import os

import xdg


class Config:
    def __init__(self, config_file):
        self._config_file = config_file
        self._config = configparser.ConfigParser()
        self._config.read(self._config_file)

    def get(self, section, option, fallback=None):
        if self._config.has_option(section, option):
            return self._config.get(section, option)
        return fallback


PACKAGE_NAME = "reddit_radio"
config_dir = xdg.xdg_config_home() / PACKAGE_NAME

config = Config(config_dir / "config.ini")

REDDIT_CONFIG = {
    "client_id": config.get("REDDIT", "client_id", ""),
    "client_secret": config.get("REDDIT", "client_secret", ""),
    "username": config.get("REDDIT", "username", ""),
    "user_agent": config.get("REDDIT", "user_agent", ""),
}
SUBREDDITS = (
    subreddits.split(",") if (subreddits := config.get("REDDIT", "subreddits")) else []
)
DATABASE = config.get(
    "DATABASE", "path", xdg.xdg_data_home() / PACKAGE_NAME / "database.db"
)
LOGS = config.get("LOGS", "path", xdg.xdg_cache_home() / PACKAGE_NAME / "{time}.log")
MPV = config.get("MPV", "path", "mpv")

if (cookies_file := config_dir / "cookies.txt").exists():
    COOKIES_FILE = cookies_file
else:
    COOKIES_FILE = None

# NOTE: override stuff for local testing, could not find a proper way to mock
# this on the tests
if os.environ.get("PYTHON_ENV") == "test":
    import tempfile

    DATABASE = tempfile.NamedTemporaryFile().name



import os
import shutil
from datetime import datetime

from reddit_radio.logging import logger


def safe_parse(parser, value, fallback=None):
    try:
        return parser(value)
    except Exception:
        logger.exception(f"Failed to parse [{value}]")
        return fallback


def fromtimestamp(timestamp):
    dt = safe_parse(datetime.fromtimestamp, timestamp)
    if not dt:
        logger.warning(f"Failed to get datetime from timestamp [{timestamp}]")
        return None
    return dt.isoformat()


def is_binary(filename):
    fpath, fname = os.path.split(filename)
    if not fpath:
        return bool(shutil.which(fname))
    return os.path.isfile(filename) and os.access(filename, os.X_OK)


class SingletonMeta(type):
    """
    From https://refactoring.guru/design-patterns/singleton/python/example
    """

    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            instance = super().__call__(*args, **kwargs)
            cls._instances[cls] = instance
        return cls._instances[cls]



from click.exceptions import UsageError
from python_mpv_jsonipc import MPV as MPVClient

from reddit_radio.config import COOKIES_FILE
from reddit_radio.config import MPV as MPV_BIN
from reddit_radio.database import RedditPost
from reddit_radio.helpers import SingletonMeta, is_binary
from reddit_radio.logging import logger


class Client(metaclass=SingletonMeta):
    def __init__(self):
        Client.check_mpv()

        kwargs = {}
        if COOKIES_FILE:
            kwargs["cookies"] = True
            kwargs["cookies_file"] = str(COOKIES_FILE)
            kwargs["ytdl_raw_options"] = f"cookies={COOKIES_FILE}"

        self._client = MPVClient(
            input_terminal=True,
            ipc_socket="/tmp/mpvsocket",
            mpv_location=MPV_BIN,
            start_mpv=True,
            terminal=True,
            video=False,
            **kwargs,
        )

    @staticmethod
    def check_mpv():
        if not is_binary(MPV_BIN):
            logger.error("mpv binary not found.")
            raise UsageError("mpv binary not found.")

    def load_playlist(self, count):
        for track in RedditPost.playlist(count):
            self._client.loadfile(track.url, "append")

    def play(self):
        index = index if (index := self._client.playlist_current_pos) >= 0 else 0
        self._client.playlist_play_index(index)

    def playlist(self, count):
        self.load_playlist(count)
        self.play()



import praw

from reddit_radio import config, youtube
from reddit_radio.helpers import SingletonMeta, fromtimestamp, safe_parse
from reddit_radio.logging import logger


class Client(metaclass=SingletonMeta):
    def __init__(self):
        self._reddit = praw.Reddit(**config.REDDIT_CONFIG)

    @staticmethod
    def serialize(post):
        return {
            "reddit_id": safe_parse(str, post.fullname, ""),
            "youtube_id": youtube.extract_video_id(post.url),
            "title": safe_parse(str, post.title, ""),
            "url": safe_parse(str, post.url, ""),
            "subreddit": safe_parse(str, post.subreddit.name, ""),
            "upvote_count": safe_parse(int, post.ups, 0),
            "upvote_ratio": safe_parse(float, post.upvote_ratio, 0),
            "submitted_at": fromtimestamp(post.created),
        }

    def get_posts(self, subreddit, method, **params):
        sub = self._reddit.subreddit(subreddit)

        try:
            args = []
            limit = params.pop("limit", 100)

            # walrus is causing black to break here
            # fmt: off
            if (time_filter := params.pop("time_filter", None)):
                args.append(time_filter)
            # fmt: on

            return list(getattr(sub, method)(*args, limit=limit, params=params))
        except Exception:
            logger.exception(f"Failed to get posts from [{subreddit}] with [{method}]")
            return []

    def get_pages(self, subreddit, method, pages=10, **params):
        for page in range(pages):
            posts = self.get_posts(subreddit, method, **params)

            if (posts_count := len(posts)) == 0 or (
                "limit" in params and params["limit"] > posts_count
            ):
                logger.info(f"{subreddit}: reached final page at [{page + 1}]")
                break

            for post in posts:
                yield self.serialize(post)

            logger.info(f"{subreddit}: {page+1:02}/{pages}")

            if (latest := posts[len(posts) - 1]) and latest.fullname:
                params["after"] = latest.fullname
            else:
                logger.info(f"{subreddit}: reached final page at [{page + 1}]")
                break

    def hot(self, subreddit, pages=10, limit=100):
        return self.get_pages(subreddit, "hot", limit=limit, pages=pages)

    def all(self, subreddit, pages=10, limit=100):
        return self.get_pages(
            subreddit, "top", time_filter="all", limit=limit, pages=pages
        )



from datetime import datetime
from pathlib import Path

import peewee

from reddit_radio import config

database = peewee.SqliteDatabase(config.DATABASE)


class BaseModel(peewee.Model):
    created_at = peewee.DateTimeField(default=datetime.now)

    class Meta:
        database = database


class RedditPost(BaseModel):
    reddit_id = peewee.CharField(unique=True)
    youtube_id = peewee.CharField(null=True)
    title = peewee.CharField()
    url = peewee.CharField(null=True)
    subreddit = peewee.CharField()
    upvote_count = peewee.IntegerField()
    upvote_ratio = peewee.FloatField()
    submitted_at = peewee.DateTimeField(null=True)

    @classmethod
    def playlist(cls, count=100):
        return (
            cls.select(cls.url)
            .where(cls.youtube_id.is_null(False) & cls.url.is_null(False))
            .order_by(peewee.fn.Random())
            .limit(count)
        )


_tables = [RedditPost]


def create_tables_if_needed():
    Path(config.DATABASE).parent.mkdir(parents=True, exist_ok=True)

    if len(database.get_tables()) == len(_tables):
        return

    with database:
        database.create_tables(_tables)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['reddit_radio']

package_data = \
{'': ['*'], 'reddit_radio': ['templates/*']}

install_requires = \
['click>=7.1.2,<8.0.0',
 'loguru>=0.5.3,<0.6.0',
 'peewee>=3.14.4,<4.0.0',
 'praw>=7.2.0,<8.0.0',
 'python-mpv-jsonipc>=1.1.13,<2.0.0',
 'xdg>=5.0.1,<6.0.0']

entry_points = \
{'console_scripts': ['reddit_radio = reddit_radio.cli:cli']}

setup_kwargs = {
    'name': 'reddit-radio',
    'version': '0.0.1',
    'description': 'Listen to music shared on reddit from your command line',
    'long_description': '# Reddit radio\n\n![workflow](https://github.com/martini97/reddit_radio/actions/workflows/ci.yaml/badge.svg)\n[![codecov](https://codecov.io/gh/martini97/reddit_radio/branch/main/graph/badge.svg)](https://codecov.io/gh/martini97/reddit_radio)\n\n## Config\n\nBefore running you need to setup the config file, there is a sample in\n`config.sample.ini` for your client\\_id and client\\_secret you need to go to this\nurl: https://www.reddit.com/prefs/apps and create a new app. Then move the file to\n`~/.config/reddit_radio.ini`.\n\n## Usage\n\n```sh\npython -m reddit_radio load-data play\n```\n\n## Dependencies\n\n+ [MPV](https://mpv.io/)\n',
    'author': 'martini97',
    'author_email': 'martini97@protonmail.ch',
    'maintainer': 'martini97',
    'maintainer_email': 'martini97@protonmail.ch',
    'url': 'https://github.com/martini97/reddit_radio',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
